@@ -28,11 +28,12 @@ import requests
 import six
 
 import gitlab.config
+from gitlab.const import *  # noqa
 from gitlab.exceptions import *  # noqa
 from gitlab.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '0.14'
+__version__ = '0.15'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -69,6 +70,8 @@ class Gitlab(object):
         user_keys (UserKeyManager): Manager for GitLab users' SSH keys.
         users (UserManager): Manager for GitLab users
         keys (DeployKeyManager): Manager for deploy keys
+        group_accessrequests (GroupAccessRequestManager): Manager for GitLab
+            groups access requests
         group_issues (GroupIssueManager): Manager for GitLab group issues
         group_projects (GroupProjectManager): Manager for GitLab group projects
         group_members (GroupMemberManager): Manager for GitLab group members
@@ -77,6 +80,8 @@ class Gitlab(object):
         issues (IssueManager): Manager for GitLab issues
         licenses (LicenseManager): Manager for licenses
         namespaces (NamespaceManager): Manager for namespaces
+        project_accessrequests (ProjectAccessRequestManager): Manager for
+            GitLab projects access requests
         project_branches (ProjectBranchManager): Manager for GitLab projects
             branches
         project_builds (ProjectBuildManager): Manager for GitLab projects
@@ -87,6 +92,8 @@ class Gitlab(object):
             GitLab projects commits comments
         project_commit_statuses (ProjectCommitStatusManager): Manager for
             GitLab projects commits statuses
+        project_deployments (ProjectDeploymentManager): Manager for GitLab
+            projects deployments
         project_keys (ProjectKeyManager): Manager for GitLab projects keys
         project_environments (ProjectEnvironmentManager): Manager for GitLab
             projects environments
@@ -101,6 +108,8 @@ class Gitlab(object):
         project_members (ProjectMemberManager): Manager for GitLab projects
             members
         project_notes (ProjectNoteManager): Manager for GitLab projects notes
+        project_pipelines (ProjectPipelineManager): Manager for GitLab projects
+            pipelines
         project_tags (ProjectTagManager): Manager for GitLab projects tags
         project_mergerequest_notes (ProjectMergeRequestNoteManager): Manager
             for GitLab notes on merge requests
@@ -111,6 +120,8 @@ class Gitlab(object):
         project_labels (ProjectLabelManager): Manager for GitLab projects
             labels
         project_files (ProjectFileManager): Manager for GitLab projects files
+        project_services (ProjectServiceManager): Manager for the GitLab
+            projects services
         project_snippet_notes (ProjectSnippetNoteManager): Manager for GitLab
             note on snippets
         project_snippets (ProjectSnippetManager): Manager for GitLab projects
@@ -124,6 +135,7 @@ class Gitlab(object):
         team_members (TeamMemberManager): Manager for GitLab teams members
         team_projects (TeamProjectManager): Manager for GitLab teams projects
         teams (TeamManager): Manager for GitLab teams
+        todos (TodoManager): Manager for user todos
     """
 
     def __init__(self, url, private_token=None, email=None, password=None,
@@ -153,6 +165,7 @@ class Gitlab(object):
         self.user_keys = UserKeyManager(self)
         self.users = UserManager(self)
         self.keys = KeyManager(self)
+        self.group_accessrequests = GroupAccessRequestManager(self)
         self.group_issues = GroupIssueManager(self)
         self.group_projects = GroupProjectManager(self)
         self.group_members = GroupMemberManager(self)
@@ -161,11 +174,13 @@ class Gitlab(object):
         self.issues = IssueManager(self)
         self.licenses = LicenseManager(self)
         self.namespaces = NamespaceManager(self)
+        self.project_accessrequests = ProjectAccessRequestManager(self)
         self.project_branches = ProjectBranchManager(self)
         self.project_builds = ProjectBuildManager(self)
         self.project_commits = ProjectCommitManager(self)
         self.project_commit_comments = ProjectCommitCommentManager(self)
         self.project_commit_statuses = ProjectCommitStatusManager(self)
+        self.project_deployments = ProjectDeploymentManager(self)
         self.project_keys = ProjectKeyManager(self)
         self.project_environments = ProjectEnvironmentManager(self)
         self.project_events = ProjectEventManager(self)
@@ -175,12 +190,14 @@ class Gitlab(object):
         self.project_issues = ProjectIssueManager(self)
         self.project_members = ProjectMemberManager(self)
         self.project_notes = ProjectNoteManager(self)
+        self.project_pipelines = ProjectPipelineManager(self)
         self.project_tags = ProjectTagManager(self)
         self.project_mergerequest_notes = ProjectMergeRequestNoteManager(self)
         self.project_mergerequests = ProjectMergeRequestManager(self)
         self.project_milestones = ProjectMilestoneManager(self)
         self.project_labels = ProjectLabelManager(self)
         self.project_files = ProjectFileManager(self)
+        self.project_services = ProjectServiceManager(self)
         self.project_snippet_notes = ProjectSnippetNoteManager(self)
         self.project_snippets = ProjectSnippetManager(self)
         self.project_triggers = ProjectTriggerManager(self)
@@ -191,6 +208,8 @@ class Gitlab(object):
         self.team_members = TeamMemberManager(self)
         self.team_projects = TeamProjectManager(self)
         self.teams = TeamManager(self)
+        self.todos = TodoManager(self)
+        self.sidekiq = SidekiqManager(self)
 
     @staticmethod
     def from_config(gitlab_id=None, config_files=None):
@@ -272,10 +291,9 @@ class Gitlab(object):
             url = obj_url % args
 
         if id_ is not None:
-            url = '%s%s/%s' % (self._url, url, str(id_))
+            return '%s/%s' % (url, str(id_))
         else:
-            url = '%s%s' % (self._url, url)
-        return url
+            return url
 
     def _create_headers(self, content_type=None, headers={}):
         request_headers = self.headers.copy()
@@ -306,8 +324,26 @@ class Gitlab(object):
         self.email = email
         self.password = password
 
+    def enable_debug(self):
+        import logging
+        try:
+            from http.client import HTTPConnection
+        except ImportError:
+            from httplib import HTTPConnection  # noqa
+
+        HTTPConnection.debuglevel = 1
+        logging.basicConfig()
+        logging.getLogger().setLevel(logging.DEBUG)
+        requests_log = logging.getLogger("requests.packages.urllib3")
+        requests_log.setLevel(logging.DEBUG)
+        requests_log.propagate = True
+
     def _raw_get(self, path, content_type=None, streamed=False, **kwargs):
-        url = '%s%s' % (self._url, path)
+        if path.startswith('http://') or path.startswith('https://'):
+            url = path
+        else:
+            url = '%s%s' % (self._url, path)
+
         headers = self._create_headers(content_type)
         try:
             return self.session.get(url,
@@ -324,23 +360,24 @@ class Gitlab(object):
                 "Can't connect to GitLab server (%s)" % e)
 
     def _raw_list(self, path, cls, extra_attrs={}, **kwargs):
-        r = self._raw_get(path, **kwargs)
-        raise_error_from_response(r, GitlabListError)
+        params = extra_attrs.copy()
+        params.update(kwargs.copy())
 
-        cls_kwargs = extra_attrs.copy()
-        cls_kwargs.update(kwargs.copy())
-
-        # Add _from_api manually, because we are not creating objects
-        # through normal path
-        cls_kwargs['_from_api'] = True
         get_all_results = kwargs.get('all', False)
 
         # Remove parameters from kwargs before passing it to constructor
         for key in ['all', 'page', 'per_page', 'sudo', 'next_url']:
-            if key in cls_kwargs:
-                del cls_kwargs[key]
+            if key in params:
+                del params[key]
 
-        results = [cls(self, item, **cls_kwargs) for item in r.json()
+        r = self._raw_get(path, **params)
+        raise_error_from_response(r, GitlabListError)
+
+        # Add _from_api manually, because we are not creating objects
+        # through normal path
+        params['_from_api'] = True
+
+        results = [cls(self, item, **params) for item in r.json()
                    if item is not None]
         if ('next' in r.links and 'url' in r.links['next']
            and get_all_results is True):
@@ -421,52 +458,8 @@ class Gitlab(object):
                                   ", ".join(missing))
 
         url = self._construct_url(id_=None, obj=obj_class, parameters=kwargs)
-        headers = self._create_headers()
 
-        # Remove attributes that are used in url so that there is only
-        # url-parameters left
-        params = kwargs.copy()
-        for attribute in obj_class.requiredUrlAttrs:
-            del params[attribute]
-
-        # Also remove the next-url attribute that make queries fail
-        if 'next_url' in params:
-            del params['next_url']
-        try:
-            r = self.session.get(url, params=params, headers=headers,
-                                 verify=self.ssl_verify,
-                                 timeout=self.timeout,
-                                 auth=requests.auth.HTTPBasicAuth(
-                                     self.http_username,
-                                     self.http_password))
-        except Exception as e:
-            raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % e)
-
-        raise_error_from_response(r, GitlabListError)
-
-        cls = obj_class
-        cls_kwargs = kwargs.copy()
-
-        # Add _from_api manually, because we are not creating objects
-        # through normal path
-        cls_kwargs['_from_api'] = True
-
-        get_all_results = params.get('all', False)
-
-        # Remove parameters from kwargs before passing it to constructor
-        for key in ['all', 'page', 'per_page', 'sudo', 'next_url']:
-            if key in cls_kwargs:
-                del cls_kwargs[key]
-
-        results = [cls(self, item, **cls_kwargs) for item in r.json()
-                   if item is not None]
-        if ('next' in r.links and 'url' in r.links['next']
-           and get_all_results is True):
-            args = kwargs.copy()
-            args['next_url'] = r.links['next']['url']
-            results.extend(self.list(obj_class, **args))
-        return results
+        return self._raw_list(url, obj_class, **kwargs)
 
     def get(self, obj_class, id=None, **kwargs):
         """Request a GitLab resources.
@@ -492,27 +485,10 @@ class Gitlab(object):
             raise GitlabGetError('Missing attribute(s): %s' %
                                  ", ".join(missing))
 
-        sanitized_id = _sanitize(id)
-        url = self._construct_url(id_=sanitized_id, obj=obj_class,
+        url = self._construct_url(id_=_sanitize(id), obj=obj_class,
                                   parameters=kwargs)
-        headers = self._create_headers()
-
-        # Remove attributes that are used in url so that there is only
-        # url-parameters left
-        params = kwargs.copy()
-        for attribute in obj_class.requiredUrlAttrs:
-            del params[attribute]
-
-        try:
-            r = self.session.get(url, params=params, headers=headers,
-                                 verify=self.ssl_verify, timeout=self.timeout,
-                                 auth=requests.auth.HTTPBasicAuth(
-                                     self.http_username,
-                                     self.http_password))
-        except Exception as e:
-            raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % e)
 
+        r = self._raw_get(url, **kwargs)
         raise_error_from_response(r, GitlabGetError)
         return r.json()
 
@@ -554,30 +530,13 @@ class Gitlab(object):
 
         obj_id = params[obj.idAttr] if obj._id_in_delete_url else None
         url = self._construct_url(id_=obj_id, obj=obj, parameters=params)
-        headers = self._create_headers()
 
-        # Remove attributes that are used in url so that there is only
-        # url-parameters left
-        for attribute in obj.requiredUrlAttrs:
-            del params[attribute]
         if obj._id_in_delete_url:
             # The ID is already built, no need to add it as extra key in query
             # string
             params.pop(obj.idAttr)
 
-        try:
-            r = self.session.delete(url,
-                                    params=params,
-                                    headers=headers,
-                                    verify=self.ssl_verify,
-                                    timeout=self.timeout,
-                                    auth=requests.auth.HTTPBasicAuth(
-                                        self.http_username,
-                                        self.http_password))
-        except Exception as e:
-            raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % e)
-
+        r = self._raw_delete(url, **params)
         raise_error_from_response(r, GitlabDeleteError)
         return True
 
@@ -612,23 +571,11 @@ class Gitlab(object):
 
         url = self._construct_url(id_=None, obj=obj, parameters=params,
                                   action='create')
-        headers = self._create_headers(content_type="application/json")
 
         # build data that can really be sent to server
         data = obj._data_for_gitlab(extra_parameters=kwargs)
 
-        try:
-            r = self.session.post(url, data=data,
-                                  headers=headers,
-                                  verify=self.ssl_verify,
-                                  timeout=self.timeout,
-                                  auth=requests.auth.HTTPBasicAuth(
-                                      self.http_username,
-                                      self.http_password))
-        except Exception as e:
-            raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % e)
-
+        r = self._raw_post(url, data=data, content_type='application/json')
         raise_error_from_response(r, GitlabCreateError, 201)
         return r.json()
 
@@ -665,158 +612,10 @@ class Gitlab(object):
                                     ", ".join(missing))
         obj_id = params[obj.idAttr] if obj._id_in_update_url else None
         url = self._construct_url(id_=obj_id, obj=obj, parameters=params)
-        headers = self._create_headers(content_type="application/json")
 
         # build data that can really be sent to server
         data = obj._data_for_gitlab(extra_parameters=kwargs, update=True)
 
-        try:
-            r = self.session.put(url, data=data,
-                                 headers=headers,
-                                 verify=self.ssl_verify,
-                                 timeout=self.timeout,
-                                 auth=requests.auth.HTTPBasicAuth(
-                                     self.http_username,
-                                     self.http_password))
-        except Exception as e:
-            raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % e)
-
+        r = self._raw_put(url, data=data, content_type='application/json')
         raise_error_from_response(r, GitlabUpdateError)
         return r.json()
-
-    def Hook(self, id=None, **kwargs):
-        """Creates/tests/lists system hook(s) known by the GitLab server.
-
-        If id is None, returns a list of hooks.
-
-        If id is an integer, tests the matching hook.
-
-        If id is a dict, creates a new object using attributes provided. The
-        object is NOT saved on the server. Use the save() method on the object
-        to write it on the server.
-        """
-        warnings.warn("`Hook` is deprecated, use `hooks` instead",
-                      DeprecationWarning)
-        return Hook._get_list_or_object(self, id, **kwargs)
-
-    def Project(self, id=None, **kwargs):
-        """Creates/gets/lists project(s) known by the GitLab server.
-
-        If id is None, returns a list of projects.
-
-        If id is an integer, returns the matching project (or raises a
-        GitlabGetError if not found)
-
-        If id is a dict, creates a new object using attributes provided. The
-        object is NOT saved on the server. Use the save() method on the object
-        to write it on the server.
-        """
-        warnings.warn("`Project` is deprecated, use `projects` instead",
-                      DeprecationWarning)
-        return Project._get_list_or_object(self, id, **kwargs)
-
-    def UserProject(self, id=None, **kwargs):
-        """Creates a project for a user.
-
-        id must be a dict.
-        """
-        warnings.warn("`UserProject` is deprecated, "
-                      "use `user_projects` instead",
-                      DeprecationWarning)
-        return UserProject._get_list_or_object(self, id, **kwargs)
-
-    def _list_projects(self, url, **kwargs):
-        r = self._raw_get(url, **kwargs)
-        raise_error_from_response(r, GitlabListError)
-
-        l = []
-        for o in r.json():
-            p = Project(self, o)
-            p._from_api = True
-            l.append(p)
-
-        return l
-
-    def search_projects(self, query, **kwargs):
-        """Searches projects by  name.
-
-        Returns a list of matching projects.
-        """
-        warnings.warn("`search_projects()` is deprecated, "
-                      "use `projects.search()` instead",
-                      DeprecationWarning)
-        return self._list_projects("/projects/search/" + query, **kwargs)
-
-    def all_projects(self, **kwargs):
-        """Lists all the projects (need admin rights)."""
-        warnings.warn("`all_projects()` is deprecated, "
-                      "use `projects.all()` instead",
-                      DeprecationWarning)
-        return self._list_projects("/projects/all", **kwargs)
-
-    def owned_projects(self, **kwargs):
-        """Lists owned projects."""
-        warnings.warn("`owned_projects()` is deprecated, "
-                      "use `projects.owned()` instead",
-                      DeprecationWarning)
-        return self._list_projects("/projects/owned", **kwargs)
-
-    def Group(self, id=None, **kwargs):
-        """Creates/gets/lists group(s) known by the GitLab server
-
-        Args:
-            id: If id is None, returns a list of groups.
-            id: If id is an integer,
-                returns the matching group (or raises a GitlabGetError if not
-                found).
-            id: If id is a dict, creates a new object using attributes
-                provided. The object is NOT saved on the server. Use the
-                save() method on the object to write it on the server.
-            kwargs: Arbitrary keyword arguments
-        """
-        warnings.warn("`Group` is deprecated, use `groups` instead",
-                      DeprecationWarning)
-        return Group._get_list_or_object(self, id, **kwargs)
-
-    def Issue(self, id=None, **kwargs):
-        """Lists issues(s) known by the GitLab server.
-
-        Does not support creation or getting a single issue unlike other
-        methods in this class yet.
-        """
-        warnings.warn("`Issue` is deprecated, use `issues` instead",
-                      DeprecationWarning)
-        return Issue._get_list_or_object(self, id, **kwargs)
-
-    def User(self, id=None, **kwargs):
-        """Creates/gets/lists users(s) known by the GitLab server.
-
-        If id is None, returns a list of users.
-
-        If id is an integer, returns the matching user (or raises a
-        GitlabGetError if not found)
-
-        If id is a dict, creates a new object using attributes provided. The
-        object is NOT saved on the server. Use the save() method on the object
-        to write it on the server.
-        """
-        warnings.warn("`User` is deprecated, use `users` instead",
-                      DeprecationWarning)
-        return User._get_list_or_object(self, id, **kwargs)
-
-    def Team(self, id=None, **kwargs):
-        """Creates/gets/lists team(s) known by the GitLab server.
-
-        If id is None, returns a list of teams.
-
-        If id is an integer, returns the matching team (or raises a
-        GitlabGetError if not found)
-
-        If id is a dict, create a new object using attributes provided. The
-        object is NOT saved on the server. Use the save() method on the object
-        to write it on the server.
-        """
-        warnings.warn("`Team` is deprecated, use `teams` instead",
-                      DeprecationWarning)
-        return Team._get_list_or_object(self, id, **kwargs)
