@@ -1,4 +1,5 @@
 import base64
+import time
 
 import gitlab
 
@@ -67,6 +68,13 @@ else:
 key = new_user.keys.create({'title': 'testkey', 'key': SSH_KEY})
 assert(len(new_user.keys.list()) == 1)
 key.delete()
+assert(len(new_user.keys.list()) == 0)
+
+# emails
+email = new_user.emails.create({'email': 'foo2@bar.com'})
+assert(len(new_user.emails.list()) == 1)
+email.delete()
+assert(len(new_user.emails.list()) == 0)
 
 new_user.delete()
 foobar_user.delete()
@@ -135,6 +143,7 @@ admin_project.files.create({'file_path': 'README',
                             'commit_message': 'Initial commit'})
 readme = admin_project.files.get(file_path='README', ref='master')
 readme.content = base64.b64encode("Improved README")
+time.sleep(2)
 readme.save(branch_name="master", commit_message="new commit")
 readme.delete(commit_message="Removing README")
 
@@ -145,13 +154,13 @@ admin_project.files.create({'file_path': 'README.rst',
 readme = admin_project.files.get(file_path='README.rst', ref='master')
 assert(readme.decode() == 'Initial content')
 
-tree = admin_project.tree()
+tree = admin_project.repository_tree()
 assert(len(tree) == 1)
 assert(tree[0]['name'] == 'README.rst')
-blob = admin_project.blob('master', 'README.rst')
+blob = admin_project.repository_blob('master', 'README.rst')
 assert(blob == 'Initial content')
-archive1 = admin_project.archive()
-archive2 = admin_project.archive('master')
+archive1 = admin_project.repository_archive()
+archive2 = admin_project.repository_archive('master')
 assert(archive1 == archive2)
 
 # labels
@@ -161,6 +170,10 @@ assert(len(admin_project.labels.list()) == 1)
 label1.new_name = 'label1updated'
 label1.save()
 assert(label1.name == 'label1updated')
+label1.subscribe()
+assert(label1.subscribed == True)
+label1.unsubscribe()
+assert(label1.subscribed == False)
 label1.delete()
 
 # milestones
@@ -208,3 +221,33 @@ v1.save()
 v1 = admin_project.variables.get(v1.key)
 assert(v1.value == 'new_value1')
 v1.delete()
+
+# branches and merges
+to_merge = admin_project.branches.create({'branch_name': 'branch1',
+                                          'ref': 'master'})
+admin_project.files.create({'file_path': 'README2.rst',
+                            'branch_name': 'branch1',
+                            'content': 'Initial content',
+                            'commit_message': 'New commit in new branch'})
+mr = admin_project.mergerequests.create({'source_branch': 'branch1',
+                                         'target_branch': 'master',
+                                         'title': 'MR readme2'})
+ret = mr.merge()
+admin_project.branches.delete('branch1')
+
+try:
+    mr.merge()
+except gitlab.GitlabMRClosedError:
+    pass
+
+# stars
+admin_project = admin_project.star()
+assert(admin_project.star_count == 1)
+admin_project = admin_project.unstar()
+assert(admin_project.star_count == 0)
+
+# namespaces
+ns = gl.namespaces.list()
+assert(len(ns) != 0)
+ns = gl.namespaces.list(search='root')[0]
+assert(ns.kind == 'user')
